#ifndef COMMAND_H_
#define COMMAND_H_

/* File Inclusions */
#include "structs.h" /* Definition of structs used by TRMOD */
#include "errors.h"  /* Error Handling for TRMOD */

/* Macro's used when an * is entered instead of a number */
#define ALL_U32 0xFFFFFFFFU
#define ALL_U16 0xFFFFU

enum commands
{
	CMD_NONE,
	
	GET_ITEM,
	CGET_ITEM,
	RGET_ITEM,
	HGET_ITEM,
	ADD_ITEM,
	REPLACE_ITEM,
	REMOVE_ITEM,
	
	GET_STATICMESH,
	CGET_STATICMESH,
	RGET_STATICMESH,
	HGET_STATICMESH,
	ADD_STATICMESH,
	REPLACE_STATICMESH,
	REMOVE_STATICMESH,
	
	GET_LIGHT,
	CGET_LIGHT,
	RGET_LIGHT,
	HGET_LIGHT,
	ADD_LIGHT,
	REPLACE_LIGHT,
	REMOVE_LIGHT,
	
	GET_SOUNDSOURCE,
	CGET_SOUNDSOURCE,
	RGET_SOUNDSOURCE,
	HGET_SOUNDSOURCE,
	ADD_SOUNDSOURCE,
	REPLACE_SOUNDSOURCE,
	REMOVE_SOUNDSOURCE,
	
	GET_FLOORDATA,
	CGET_FLOORDATA,
	RGET_FLOORDATA,
	REPLACE_FLOORDATA,
	REMOVE_FLOORDATA,
	
	GET_FLOOR,
	CGET_FLOOR,
	RGET_FLOOR,
	HGET_FLOOR,
	SET_FLOOR,
	
	GET_CEILING,
	CGET_CEILING,
	RGET_CEILING,
	HGET_CEILING,
	SET_CEILING,
	
	GET_ZONE,
	CGET_ZONE,
	RGET_ZONE,
	SET_ZONE,
	
	GET_STEPSOUND,
	CGET_STEPSOUND,
	RGET_STEPSOUND,
	SET_STEPSOUND,
	
	GET_VERTEX,
	CGET_VERTEX,
	RGET_VERTEX,
	HGET_VERTEX,
	ADD_VERTEX,
	REPLACE_VERTEX,
	REMOVE_VERTEX,
	REMOVE_GEOMETRY,
	
	GET_RECTANGLE,
	CGET_RECTANGLE,
	RGET_RECTANGLE,
	HGET_RECTANGLE,
	ADD_RECTANGLE,
	REPLACE_RECTANGLE,
	REMOVE_RECTANGLE,
	
	GET_TRIANGLE,
	CGET_TRIANGLE,
	RGET_TRIANGLE,
	HGET_TRIANGLE,
	ADD_TRIANGLE,
	REPLACE_TRIANGLE,
	REMOVE_TRIANGLE,
	
	GET_SPRITE,
	CGET_SPRITE,
	RGET_SPRITE,
	HGET_SPRITE,
	ADD_SPRITE,
	REPLACE_SPRITE,
	REMOVE_SPRITE,
	
	GET_VIEWPORT,
	CGET_VIEWPORT,
	RGET_VIEWPORT,
	ADD_VIEWPORT,
	REPLACE_VIEWPORT,
	REMOVE_VIEWPORT,
	
	GET_ROOMLIGHT,
	CGET_ROOMLIGHT,
	RGET_ROOMLIGHT,
	SET_ROOMLIGHT,
	
	GET_ALTROOM,
	CGET_ALTROOM,
	RGET_ALTROOM,
	SET_ALTROOM,
	
	GET_ROOMFLAGS,
	CGET_ROOMFLAGS,
	RGET_ROOMFLAGS,
	
	GET_WATER,
	CGET_WATER,
	RGET_WATER,
	SET_WATER_ON,
	SET_WATER_OFF,
	
	GET_SFXALWAYS,
	CGET_SFXALWAYS,
	RGET_SFXALWAYS,
	SET_SFXALWAYS_ON,
	SET_SFXALWAYS_OFF,
	
	GET_PITCHSHIFT,
	CGET_PITCHSHIFT,
	RGET_PITCHSHIFT,
	SET_PITCHSHIFT_ON,
	SET_PITCHSHIFT_OFF,
	
	GET_SKY,
	CGET_SKY,
	RGET_SKY,
	SET_SKY_ON,
	SET_SKY_OFF,
	
	GET_DYNAMIC,
	CGET_DYNAMIC,
	RGET_DYNAMIC,
	SET_DYNAMIC_ON,
	SET_DYNAMIC_OFF,
	
	GET_WIND,
	CGET_WIND,
	RGET_WIND,
	SET_WIND_ON,
	SET_WIND_OFF,
	
	GET_INSIDE,
	CGET_INSIDE,
	RGET_INSIDE,
	SET_INSIDE_ON,
	SET_INSIDE_OFF,
	
	GET_SWAMP,
	CGET_SWAMP,
	RGET_SWAMP,
	SET_SWAMP_ON,
	SET_SWAMP_OFF,
	
	GET_ROOMPOS,
	CGET_ROOMPOS,
	RGET_ROOMPOS,
	MOVE_ROOM,
	
	GET_ROOMSIZE,
	CGET_ROOMSIZE,
	RGET_ROOMSIZE,
	RESIZE_ROOM,
	
	GET_BOX,
	CGET_BOX,
	RGET_BOX,
	ADD_BOX,
	REPLACE_BOX,
	REMOVE_ALL_BOXES,
	
	ADD_ROOM,
	REMOVE_ALL_ROOMS,
	HEXDUMP,
	VARDUMP,
	
	NUM_COMMANDS,
	
	CMD_INVALID
};

void interpretCommand(struct level *level, struct error *error, char *in);
void showUsage(char *arg0);

#endif
